//
// This code is Copyright  2007 Colin Angus Mackay. All Rights Reserved.
// 
// This code was created for educational purposes, but it may be used on the 
// following conditions.
//
// 1. All copyright notices remain intact, including this one.
// 2. This license remains intact.
// 3. If the code is used in any software the copyright notice must be replicated
//    in the "About" box or equivalent location. i.e. anywhere the software's
//    copyright notice appears.
// 4. If the code is extracted into another assembly then the 
//    AssemblyCopyrightAttribute for the assembly must include the copyright 
//    notice "Parts of this software are Copyright  2007 Colin Angus Mackay."
// 5. This code is provided "AS IS" without warranty of any kind. You may use
//    the code only on the understanding that you use it at your own risk.
//
// For more information see http://www.colinmackay.net
//
using System;
using System.Collections.Generic;
using System.Text;

namespace ColinAngusMackay.NumbersInWords
{
        public class English
        {
            public static string NumberToEnglish(int number)
            {
                // Special case for zero
                if (number == 0)
                    return "zero";

                // Special case for int.MinValue.
                if (number == int.MinValue)
                    return "negative two billion one hundred and forty seven million " +
                        "four hundred and eighty three thousand six hundred and forty eight";

                // set up possible prefix for negative numbers
                string prefix = "";
                if (number < 0)
                    prefix = "negative ";


                int absNumber = Math.Abs(number);

                return string.Concat(prefix,
                    TranslateOneToTwoBillionOneHundredAndFortySevenMillionFourHundredAndEightyThreeThousandSixHundredAndFortySeven(absNumber));
            }

            private static string TranslateOneToTwoBillionOneHundredAndFortySevenMillionFourHundredAndEightyThreeThousandSixHundredAndFortySeven(int number)
            {
                if (number < 1000000000)
                    return TranslateOneToNineHundredAndNintyNineMillionNineHundredAndNintyNineThousandNineHundredAndNintyNine(number);

                int billions = number / 1000000000;
                string result = string.Concat(TranslateOneToNineteen(billions), " billion");

                int remainder = number % 1000000000;
                if (remainder == 0)
                    return result;

                if (remainder < 100)
                    return string.Concat(result, " and ", TranslateOneToNinetyNine(remainder));

                return string.Concat(result, " ",
                    TranslateOneToNineHundredAndNintyNineMillionNineHundredAndNintyNineThousandNineHundredAndNintyNine(remainder));
            }

            private static string TranslateOneToNineHundredAndNintyNineMillionNineHundredAndNintyNineThousandNineHundredAndNintyNine(int number)
            {
                if (number < 1000000)
                    return TranslateOneToNineHundredAndNinetyNineThousandNineHundredAndNinetyNine(number);

                int millions = number / 1000000;
                string result = string.Concat(TranslateOneToNineHundredAndNinetyNine(millions),
                    " million");
                int remainder = number % 1000000;
                if (remainder == 0)
                    return result;

                if (remainder < 100)
                    return string.Concat(result, " and ",
                        TranslateOneToNinetyNine(remainder));

                return string.Concat(result, " ",
                    TranslateOneToNineHundredAndNinetyNineThousandNineHundredAndNinetyNine(remainder));
            }

        private static string TranslateOneToNineHundredAndNinetyNineThousandNineHundredAndNinetyNine(int number)
        {
            if (number < 1000)
                return TranslateOneToNineHundredAndNinetyNine(number);

            int thousands = number / 1000;
            string result = string.Concat(TranslateOneToNineHundredAndNinetyNine(thousands),
                " thousand");
            int remainder = number % 1000;
            if (remainder == 0)
                return result;

            if (remainder < 100)
                return string.Concat(result, " and ",
                    TranslateOneToNinetyNine(remainder));

            return string.Concat(result, " ",
                TranslateOneToNineHundredAndNinetyNine(remainder));
        }

        private static string TranslateOneToNineHundredAndNinetyNine(int number)
        {
            if (number < 100)
                return TranslateOneToNinetyNine(number);
            int hundreds = number / 100;
            string result = string.Concat(TranslateOneToNineteen(hundreds),
                " hundred");
            int remainder = number % 100;
            if (remainder == 0)
                return result;

            return string.Concat(result, " and ", TranslateOneToNinetyNine(remainder));
        }
        
        private static string TranslateOneToNinetyNine(int number)
        {
            if (number < 20)
                return TranslateOneToNineteen(number);
            int units = number % 10;
            int tens = number / 10;
            string result = "";
            switch (tens)
            {
                case 2:
                    result = "twenty";
                    break;
                case 3:
                    result = "thirty";
                    break;
                case 4:
                    result = "forty";
                    break;
                case 5:
                    result = "fifty";
                    break;
                case 6:
                    result = "sixty";
                    break;
                case 7:
                    result = "seventy";
                    break;
                case 8:
                    result = "eighty";
                    break;
                default:
                    result = "ninety";
                    break;
            }
            if (units != 0)
                result = string.Concat(result, " ", TranslateOneToNineteen(units));
            return result;
        }

        private static string TranslateOneToNineteen(int number)
        {
            switch (number)
            {
                case 1:
                    return "one";
                case 2:
                    return "two";
                case 3:
                    return "three";
                case 4:
                    return "four";
                case 5:
                    return "five";
                case 6:
                    return "six";
                case 7:
                    return "seven";
                case 8:
                    return "eight";
                case 9:
                    return "nine";
                case 10:
                    return "ten";
                case 11:
                    return "eleven";
                case 12:
                    return "twelve";
                case 13:
                    return "thirteen";
                case 14:
                    return "fourteen";
                case 15:
                    return "fifteen";
                case 16:
                    return "sixteen";
                case 17:
                    return "seventeen";
                case 18:
                    return "eighteen";
                default:
                    return "nineteen";
            }
        }
    }
}
